/* filter.h - filter definitions
 * Rob Earhart
 * $Id$
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef FILTER_H
#define FILTER_H

#define NEW(W, X) do {				\
  (W) = (X *)(malloc(sizeof(X)));		\
  ((Base *)(W))->ops = (BaseOps *)&(X ## Ops);	\
} while(0)

struct base_s;

typedef struct {
  void (*print)(struct base_s *it, FILE *stream);
} BaseOps;

#define base_Print(x, stream) \
  (((Base *)(x))->ops->print((Base *)(x), (stream)))

typedef struct base_s {
  BaseOps *ops;
} Base;

typedef struct list_s {
  Base base;
  struct list_s *next;
} List;

typedef struct strings_s {
  List list;
  char *str;
} Strings;

extern BaseOps StringsOps;

struct test_s;

typedef struct {
  BaseOps baseops;
  int (*eval)(struct test_s *test);
} TestOps;

#define test_Eval(x) \
  (((TestOps *)((Base *)(x))->ops)->eval((Test *)(x)))

typedef struct test_s {
  Base *base;
} Test;

typedef struct tests_s {
  List list;
  Test *test;
} Tests;

extern TestOps TestsOps;

typedef struct all_of_s {
  Test test;
  Tests *tests;
} All_Of;

extern TestOps All_OfOps;

typedef struct any_of_s {
  Test test;
  Tests *tests;
} Any_Of;

extern TestOps Any_OfOps;

typedef struct exists_s {
  Test test;
  Strings *headers;
} Exists;

extern TestOps ExistsOps;

typedef struct false_s {
  Test test;
} False;

extern TestOps FalseOps;

typedef struct header_s {
  Test test;
  Strings *headers;
  int matcher;
  Strings *keys;
} Header;

extern TestOps HeaderOps;

typedef struct not_s {
  Test test;
  Test *it;
} Not;

extern TestOps NotOps;

typedef struct size_s {
  Test test;
  int over;
  unsigned value;
} Size;

extern TestOps SizeOps;

typedef struct support_s {
  Test test;
  char *extension;
} Support;

extern TestOps SupportOps;

typedef struct true_s {
  Test test;
} True;

extern TestOps TrueOps;

struct cmd_s;

typedef struct {
  BaseOps baseops;
  void (*eval)(struct cmd_s *cmd);
} CmdOps;

#define cmd_Eval(x) \
  (((CmdOps *)((Base *)(x))->ops)->eval((Cmd *)(x)))

typedef struct cmd_s {
  List list;
} Cmd;

typedef struct clause_s {
  Cmd cmd;
  Test *test;
  Cmd *body;
} Clause;

extern CmdOps ClauseOps;

typedef struct if_s {
  Cmd cmd;
  Clause *clause;
} If;

extern CmdOps IfOps;

typedef struct require_s {
  Cmd cmd;
  char *ext;
} Require;

extern CmdOps RequireOps;

typedef struct bounce_s {
  Cmd cmd;
} Bounce;

extern CmdOps BounceOps;

typedef struct fileinto_s {
  Cmd cmd;
  char *folder;
} FileInto;

extern CmdOps FileIntoOps;

typedef struct forward_s {
  Cmd cmd;
  char *where;
} Forward;

extern CmdOps ForwardOps;

typedef struct normal_s {
  Cmd cmd;
} Normal;

extern CmdOps NormalOps;

typedef struct reply_s {
  Cmd cmd;
  char *msg;
} Reply;

extern CmdOps ReplyOps;

typedef struct stop_s {
  Cmd cmd;
} Stop;

extern CmdOps StopOps;

typedef struct toss_s {
  Cmd cmd;
} Toss;

extern CmdOps TossOps;

#endif /* FILTER_H */
